@@ -23,7 +23,6 @@
 #ifndef __GDL_H__
 #define __GDL_H__
 
-#include <gdl/GDL.h>
 #include <gdl/gdl-tools.h>
 #include <gdl/gdl-dock-object.h>
 #include <gdl/gdl-dock-master.h>
@@ -33,8 +32,7 @@
 #include <gdl/gdl-dock-paned.h>
 #include <gdl/gdl-dock-notebook.h>
 #include <gdl/gdl-dock-tablabel.h>
-#include <gdl/gdl-recent.h>
-#include <gdl/gdl-icons.h>
+#include <gdl/gdl-dock-bar.h>
 #include <gdl/gdl-combo-button.h>
 
 #endif
